// 
// LipoMètre pour télémétrie Graupner Hott
// 
// Version 0.35 
//
//
// Auteur : www.thyzoon.fr 
//
// 2011
//

//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
#include "GMessage.h"
#include "Tension_Lipo.h"

// Def LED verte carte Arduino mini pro
#define LEDPIN_PINMODE          pinMode (13, OUTPUT);
#define LEDPIN_SWITCH           PINB |= 1<<5;     //switch LEDPIN state (digital PIN 13)
#define LEDPIN_OFF              PORTB &= ~(1<<5);
#define LEDPIN_ON               PORTB |= (1<<5);

// Baudrate UART
#define SERIAL_COM_SPEED 		19200

// Gestion jauge (seuil de tension)
#define T75P 3.92
#define T50P 3.8 
#define T25P 3.66
#define T0P  3.5


// Variables
static uint8_t octet1 = 0;  // reception
static uint8_t octet2 = 0;
static uint8_t nb_Lipo = 3; // Nb d'éléments accu Lipo
float  Lipo_total =0;   // Tension totale mesurée
float lipo1 = 0.0; // Tension sur chacun des éléments
float lipo2 = 0.0;
float lipo3 = 0.0;
float lipo4 = 0.0; 
float lipo5 = 0.0; 
float lipo6 = 0.0; 
float lipo_moyen = 0.0; 
uint8_t Jauge = 0;

// 
GMessage message;
TensionLipo tension;

// Init
void setup() {
  Serial.begin(SERIAL_COM_SPEED);  // Initialisation COM 19200 bauds pour recepteur HOTT GRxx
  tension.begin();
  LEDPIN_PINMODE
  LEDPIN_OFF 
  
  // Init. pour certaines données  
  message.setLipo1 (lipo1);
  message.setLipo2 (lipo2);
  message.setLipo3 (lipo3);      
  message.setLipo4 (lipo4);  
  message.setLipo5 (lipo5);  
  message.setLipo6 (lipo6);  
  message.setAltitudeInMeters (0);
  message.setClimbrate (0);
  
  nb_Lipo = tension.Detect (); // detection du nombre d'element 2S a 6S  
  message.setTemp1 (nb_Lipo);
  
  // Vide le buffer de réception
  Serial.flush();  
}

// Boucle pricipale
void loop() {
 
  // Reception ?
  if (Serial.available() == 2) { // si données disponibles sur le port série
    // lit 2 octets reçu
    octet1 = Serial.read();
    octet2 = Serial.read();
    // Demande RX Module General Air =	$80 $8d  (128)  (141)
    if ((octet1 == 128) && (octet2 == 141)) {
      LEDPIN_ON	   
      delay(2);

      // Mesure des tensions
      lipo1 = tension.Tension_Lipo1();  
      lipo2 = tension.Tension_Lipo2();  
      lipo3 = tension.Tension_Lipo3();  
      lipo4 = tension.Tension_Lipo4();
      lipo5 = tension.Tension_Lipo5();
      lipo6 = tension.Tension_Lipo6();
  
      // Calcul de la valeur de chaque elements
      switch (nb_Lipo){
        case 2 :
        Lipo_total = lipo2;
        lipo2 = lipo2 - lipo1;
        break;  

        case 3 :
        Lipo_total = lipo3;
        lipo3 = lipo3 - lipo2;
        lipo2 = lipo2 - lipo1;       
        message.setLipo3 (lipo3);   
        break;        

        case 4 :
        Lipo_total = lipo4;
        lipo4 = lipo4 - lipo3;
        lipo3 = lipo3 - lipo2;
        lipo2 = lipo2 - lipo1;
        message.setLipo3 (lipo3);      
        message.setLipo4 (lipo4);  
        break;  
        
        case 5 :
        Lipo_total = lipo5;
        lipo5 = lipo5 - lipo4;
        lipo4 = lipo4 - lipo3;
        lipo3 = lipo3 - lipo2;
        lipo2 = lipo2 - lipo1;
        message.setLipo3 (lipo3);    
        message.setLipo4 (lipo4);      
        message.setLipo5 (lipo5);  
        break;          
        
        case 6 :
        Lipo_total = lipo6;
        lipo6 = lipo6 - lipo5;
        lipo5 = lipo5 - lipo4;
        lipo4 = lipo4 - lipo3;
        lipo3 = lipo3 - lipo2;
        lipo2 = lipo2 - lipo1;
        message.setLipo3 (lipo3);    
        message.setLipo4 (lipo4);      
        message.setLipo5 (lipo5);  
        message.setLipo6 (lipo6);  
        break;         
        
        default : ;
      }     
         
      // par défaut 1 et 2      
      message.setLipo1 (lipo1);
      message.setLipo2 (lipo2);

      message.setVoltage1(Lipo_total);    // Affichage sur Batt 1
      message.setMainVoltage(Lipo_total); // Affichage sur principal
      
	  // Calcul de la tension moyenne
      lipo_moyen = Lipo_total / nb_Lipo;
      Jauge = 99; // Maxi.
      if (lipo_moyen < T75P) {
        Jauge = 74; }  
      if (lipo_moyen < T50P) {
          Jauge = 74; }
      if (lipo_moyen < T25P) {
            Jauge = 24; }
      if (lipo_moyen < T0P) {
            Jauge = 0; }
      
	  // Affichage jauge carburant
      message.setFuelPercent (Jauge);

      message.send();  
      delay(10);
      Serial.flush();
      LEDPIN_OFF
    }	
  }
}


